//+------------------------------------------------------------------+
//|                                                     SignalWZ.mqh |
//|                   Copyright 2009-2017, MetaQuotes Software Corp. |
//|                                              http://www.mql5.com |
//+------------------------------------------------------------------+
#include <Expert\ExpertSignal.mqh>
#include <my\Cnewton.mqh>
// wizard description start
//+------------------------------------------------------------------+
//| Description of the class                                         |
//| Title=Signals based on SVM with newton's polyomial v1            |
//| Type=SignalAdvanced                                              |
//| Name=SVM                                                         |
//| ShortName=SVM                                                    |
//| Class=CSignalSVM                                                 |
//| Page=signal_svm                                                  |
//| Parameter=Length,int,10,Length                                   |
//+------------------------------------------------------------------+
// wizard description end
//+------------------------------------------------------------------+
//| Class CSignalSVM.                                                |
//| Purpose: Class of of trade signals based on SVM with newton's    |
//|            polyomial v1                                          |
//|            Derives from class CExpertSignal.                     |
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
class CSignalSVM                       : public CExpertSignal
{
protected:

   int                                 m_length;
   Cnewton                             m_newton;

public:

   //--- method of verification of settings
   virtual bool                        ValidationSettings(void);
   //--- method of creating the indicator and timeseries
   virtual bool                        InitIndicators(CIndicators *indicators);
   //--- methods of checking if the market models are formed
   virtual int                         LongCondition(void);
   virtual int                         ShortCondition(void);
   
   struct Smodel
   {  matrix                           x;
      vector                           y;
      int                              y0s;
      int                              y1s;
                                       Smodel()
                                       {  y0s = 0;
                                          y1s = 0;
                                       };
                                       ~Smodel(){};
   };
   
   Smodel                              m_model;
   
   const int                           Dimensions(){ return(2); };

   void                                Length(int value)
   {  m_length = value;
   };

   double                              GetOutput(int Index);
   double                              SetOutput();
   
   double                              Regressor(vector &X,vector &Y,double XX,double YY);

                     CSignalSVM(void);
                    ~CSignalSVM(void);

};
//+------------------------------------------------------------------+
//| Constructor                                                      |
//+------------------------------------------------------------------+
void CSignalSVM::CSignalSVM(void)  :   m_length(10)
{
//--- initialization of protected data
   m_used_series = USE_SERIES_OPEN + USE_SERIES_HIGH + USE_SERIES_LOW + USE_SERIES_CLOSE + USE_SERIES_SPREAD + USE_SERIES_TIME;
}
//+------------------------------------------------------------------+
//| Validation settings protected data.                              |
//+------------------------------------------------------------------+
bool CSignalSVM::ValidationSettings(void)
{  if(!CExpertSignal::ValidationSettings())
      return(false);
   m_model.x.Init(m_length,Dimensions());
   m_model.y.Init(m_length);
//--- ok
   return(true);
}
//+------------------------------------------------------------------+
//| Create indicators.                                               |
//+------------------------------------------------------------------+
bool CSignalSVM::InitIndicators(CIndicators *indicators)
{
//--- check pointer
   if(indicators == NULL)
      return(false);
//--- initialization of indicators and timeseries of additional filters
   if(!CExpertSignal::InitIndicators(indicators))
      return(false);
//--- ok
   return(true);
}
//+------------------------------------------------------------------+
//| "Voting" that price will grow.                                   |
//+------------------------------------------------------------------+
int CSignalSVM::LongCondition(void)
{  int _result = 0;
   
   double _o=GetOutput(0);
//
   //PrintFormat(__FUNCSIG__+" o is: %.4f ",_o);
   if(_o > 0.0)
   {  _result = int(round(_o));
   }
//
   return(_result);
}
//+------------------------------------------------------------------+
//| "Voting" that price will fall.                                   |
//+------------------------------------------------------------------+
int CSignalSVM::ShortCondition(void)
{  int _result = 0;
   
   double _o=GetOutput(0);
//
   //PrintFormat(__FUNCSIG__+" o is: %.4f ",_o);
   if(_o < 0.0)
   {  _result = int(round(_o));
      _result *= -1;
   }
//
   return(_result);
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
double CSignalSVM::GetOutput(int Index)
{  double _get = 0.0;
   m_open.Refresh(-1);
   m_high.Refresh(-1);
   m_low.Refresh(-1);
   m_close.Refresh(-1);
   m_time.Refresh(-1);//;
   int _x = StartIndex()+Index;
   for(int i = 0; i < m_length; i++)
   {  for(int ii = 0; ii < Dimensions(); ii++)
      {  if(ii==0)//dim-1
         {  m_model.x[i][ii] = m_high.GetData(StartIndex() + i + _x) - m_high.GetData(StartIndex() + i + _x + 1);
         }
         else if(ii==1)//dim-2
         {  m_model.x[i][ii] = m_low.GetData(StartIndex() + i + _x) - m_low.GetData(StartIndex() + i + _x + 1);
         }
      }
      if(i > 0) //assign classifier 
      {  if(m_close.GetData(StartIndex() + i + _x - 1) - m_close.GetData(StartIndex() + i + _x) > 0.0)
         {  m_model.y[i - 1] = 1;
            m_model.y1s++;
         }
         else if(m_close.GetData(StartIndex() + i + _x - 1) - m_close.GetData(StartIndex() + i + _x) < 0.0)
         {  m_model.y[i - 1] = 0;
            m_model.y0s++;
         }
      }
   }
   //
   _get = SetOutput();
   
   return(_get);
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
double CSignalSVM::SetOutput(void)
{  double _set = 0.0;
   if(m_model.y0s*m_model.y1s > 0)
   {  int _index = 0;
      matrix _interpolate;
      _interpolate.Init(m_model.y0s*m_model.y1s,Dimensions());
      for(int i=0;i<m_length;i++)
      {  if(m_model.y[i] == 0)
         {  for(int ii=0;ii<m_length;ii++)
            {  if(m_model.y[ii] == 1)
               {  _interpolate[_index][0] = 0.5*(m_model.x[i][0]+m_model.x[ii][0]);
                  _interpolate[_index][1] = 0.5*(m_model.x[i][1]+m_model.x[ii][1]); 
                  _index++;
                  if(_index>=m_model.y0s*m_model.y1s)
                  {  break;
                  }
               }
            }
         }
         if(_index>=m_model.y0s*m_model.y1s)
         {  break;
         }
      }
      vector _w;
      vector _x = _interpolate.Col(0);
      vector _y = _interpolate.Col(1);
      _w.Init(m_model.y0s*m_model.y1s);
      _w[0] = _y[0];
      m_newton.Set(_w, _x, _y);
      double _xx=m_model.x[0][0],_yy=m_model.x[0][1],_zz=0.0;
      m_newton.Get(_w, _xx, _zz);
      if(_yy < _zz)
      {  _set = 100.0;
      }
      else if(_yy > _zz)
      {  _set = -100.0;
      }
      _set*=Regressor(_x,_y,_xx,_yy);
   }
   return(_set);
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
double CSignalSVM::Regressor(vector &X,vector &Y,double XX,double YY)
{
   double _x_max=fmax(X.Max(),XX);
   double _x_min=fmin(X.Min(),XX);
   double _y_max=fmax(Y.Max(),YY);
   double _y_min=fmin(Y.Min(),YY);
   return(0.5*((1.0-((_x_max-XX)/fmax(m_symbol.Point(),_x_max-_x_min)))+(1.0-((_y_max-YY)/fmax(m_symbol.Point(),_y_max-_y_min)))));
}
//+------------------------------------------------------------------+
//| Destructor                                                       |
//+------------------------------------------------------------------+
void CSignalSVM::~CSignalSVM(void)
{
}
//+------------------------------------------------------------------+
